<?php
/*
 * Syntaxe:
 *  wget -q -O - http://preconf.dr15.cnrs.fr/copie-preconf.php
 *
 * Description:
 *  Remplace le contenu du site par celui issu des fichiers copie-preconf.tar
 *  et copie-preconf.sql
 *
 *  Avant de lancer le script, effectuer les opérations suivantes :
 *  - copier par SFTP dans le répertoire tmp du site cible le fichier
 *    copie-preconf.tar issu de la sauvegarde du site preconf.dr15.cnrs.fr
 *  - copier par SFTP dans le répertoire tmp du site cible le fichier
 *    copie-preconf.sql issu de la sauvegarde du site preconf.dr15.cnrs.fr
 *  - modifier les deux constantes AUTHORIZED_NETWORK et AUTHORIZED_MASK
 *    dans ce script pour indiquer les clients autorisés à l'exécuter
 *  - indiquer dans les constantes DATABASE_* les paramètres du site cible
 *  - copier par SFTP ce script à la racine du site (répertoire
 *    monsite.cnrs.fr/www)
 *
 *  Ensuite, exécuter le script via l'URL indiquée ci-dessus.
 *
 *  Il effectue les opérations suivantes :
 *  - effacement de tous les fichiers du site
 *  - extraction du contenu de tmp/copie-preconf.tar
 *  - remplacement de toutes les occurences du nom du site de référence
 *    par le nom du site cible dans la sauvegarde de la base de données
 *  - effacement du contenu de la base de données du site cible et
 *    réimport des données
 *  - mise à jour de wp-config.php
 *  - effacement du fichier copie-preconf.php et copie-preconf.sql
 * 
 * Auteur:
 *  Roland Dirlewanger, CNRS, 2021.
 */

########################################################################
#                                                                      #
# MODIFIER LES CONSTANTES CI-DESSOUS EN FONCTION DU SITE CIBLE         #
#                                                                      #
########################################################################

# Les adresses IP autorisées
define("AUTHORIZED_NETWORK", "147.210.72.0");
define("AUTHORIZED_MASK", 0xffffff00);

# Le nom et l'identifiant du site cible
define("TARGET_NAME", "sandbox.dr15.cnrs.fr");
define("TARGET_ID", "lab9999");

# Les identifiants et mots de passe de la base de données
define("DATABASE_HOST", "mysql2.lamp.ods");
define("DATABASE_NAME", "lab9999sql0db");
define("DATABASE_USER", "lab9999sql0");
define("DATABASE_PASS", "LeMotDePasse");

########################################################################
#                                                                      #
# FIN DES MODIFICATIONS
#                                                                      #
########################################################################

define("SOURCE_ARCHIVE", "../../tmp/copie-preconf.tar");
define("SOURCE_DATABASE", "../../tmp/copie-preconf.sql");
define("SOURCE_NAME", "preconf.dr15.cnrs.fr");
define("SOURCE_ID", "lab0150");

Header("Content-type: text/plain; charset=UTF-8");

try {
    // Vérifie la provenance de la requête
    $myip = ip2long($_SERVER['HTTP_X_FORWARDED_FOR']) & AUTHORIZED_MASK;
    $authip = ip2long(AUTHORIZED_NETWORK);
    if ($myip !== $authip) {
        echo "Accès interdit\n";
        exit(1);
    }

    # Vérifie que l'archive est présente
    if (! file_exists(SOURCE_ARCHIVE)) {
        echo "Sauvegarde du site preconf absente\n";
        exit(1);
    }

    # Vérifie que la base de données est présente
    if (! file_exists(SOURCE_DATABASE)) {
        echo "Sauvegarde de la base de donnnées absente\n";
        exit(1);
    }

    # Vide le répertoire courant
    recursiveRmdir();

    # Extrait le contenu de l'archive
    $phar = new PharData(SOURCE_ARCHIVE);
    $phar->extractTo(".");

    # Récupère le contenu de la base de données de référence
    # et remplace le nom du site de référence par le nom
    # du site cible. Idem pour leurs identifiants
    $sql_content = file_get_contents(SOURCE_DATABASE);
    $sql_content = str_replace(SOURCE_NAME, TARGET_NAME, $sql_content);
    $sql_content = str_replace(SOURCE_ID, TARGET_ID, $sql_content);

    # Restaure la base de données :
    # - efface toutes les tables
    # - importe le contenu de la base
    $host = DATABASE_HOST;
    $base = DATABASE_NAME;
    $pdo = new PDO("mysql:host=$host;dbname=$base", DATABASE_USER, DATABASE_PASS);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

    // récupère la liste des tables
    $query = "SELECT table_name FROM information_schema.tables WHERE table_schema = '$base'";
    $stmt = $pdo->query($query, PDO::FETCH_COLUMN, 0);
    $tables = $stmt->fetchAll(); #PDO::FETCH_COLUMN, 0);

    // Efface toutes les tables
    $pdo->query("SET FOREIGN_KEY_CHECKS = 0");
    foreach ($tables as $table) {
        $query = "DROP TABLE IF EXISTS `$table`";
        $pdo->query($query);
    }
    $pdo->query("SET FOREIGN_KEY_CHECKS = 1");

    // Importe le contenu de la base
    $pdo->query($sql_content);

    # Met à jour le fichier wp-config.php
    $host = DATABASE_HOST;
    $name = DATABASE_NAME;
    $user = DATABASE_USER;
    $pass = DATABASE_PASS;
    $wp_config = file_get_contents("wp-config.php");
    $wp_config = preg_replace("/'DB_HOST', '.*'/U", "'DB_HOST', '$host'", $wp_config);
    $wp_config = preg_replace("/'DB_NAME', '.*'/U", "'DB_NAME', '$name'", $wp_config);
    $wp_config = preg_replace("/'DB_USER', '.*'/U", "'DB_USER', '$user'", $wp_config);
    $wp_config = preg_replace("/'DB_PASSWORD', '.*'/U", "'DB_PASSWORD', '$pass'", $wp_config);
    file_put_contents("wp-config.php", $wp_config);

    # Supprime les fichiers temporaires. Le script lui-même a été supprimé
    # lors de la suppression récursive de tous les fichiers du site
    unlink(SOURCE_ARCHIVE);
    unlink(SOURCE_DATABASE);
} catch (Exception $e) {
    echo $e->getMessage()."\n";
    echo $e->getTraceAsString();
    exit(1);
}
echo "Installation effectuée\n";

/*
 * Fonctions auxilliaires
 */

/**
 * Efface récursivement le répertoire indiqué
 *
 * @param $dir Répertoire à supprimer
 */
function recursiveRmdir($dir = ".") {
    # Récupère toutes les entrées dans le répertoire
    if (($entries = scandir($dir)) === false) {
        throw new Exception("Error while scanning $dir");
    }

    # Traite chaque entrée selon son type
    foreach ($entries as $entry) {
        # Ignore . et ..
        if ($entry == "." or $entry == "..") {
            continue;
        }
        
        # Si l'entrée est un répertoire, on le parcours récursivement
        $entry = "$dir/$entry";
        if (is_dir($entry) and !is_link($entry)) {
            recursiveRmdir($entry);
            continue;
        }

        # Dans les autres cas, on efface le fichier
        if (! unlink($entry)) {
            throw new Exception("Error while removing $entry");
        }
    }

    # Efface le répertoire lui-même sauf s'il s'agit de .
    if ($dir != ".") {
        if (! rmdir($dir)) {
            throw new Exception("Error while removing directory $dir");
        }
    }
}
?>
